#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util = require('util');
const now = require('nano-time');
const crc = require('node-crc');
const reverse = require('buffer-reverse');
const server_1 = require("./server");
const SerialPort = require('serialport');
function sendToSerialPort(port, packet) {
    let rb = packet.dataFrame.rigidBodies[0];
    let tsBuf = new Buffer(8);
    tsBuf.writeDoubleLE(packet.dataFrame.timestamp, 0);
    let rbBuf = new Buffer(28);
    rbBuf.writeFloatLE(rb.x, 0);
    rbBuf.writeFloatLE(rb.y, 4);
    rbBuf.writeFloatLE(rb.z, 8);
    rbBuf.writeFloatLE(rb.qx, 12);
    rbBuf.writeFloatLE(rb.qy, 16);
    rbBuf.writeFloatLE(rb.qz, 20);
    rbBuf.writeFloatLE(rb.qw, 24);
    let headBuf = new Buffer(2);
    headBuf.write('@', 0);
    let bodyLength = tsBuf.length + rbBuf.length;
    headBuf.writeUInt8(bodyLength + 4, 1);
    let buf = Buffer.concat([headBuf, tsBuf, rbBuf], bodyLength + headBuf.length);
    let checksum = reverse(crc.crc32(buf));
    let bufWithChecksum = Buffer.concat([buf, checksum], buf.length + checksum.length);
    port.write(bufWithChecksum);
}
const argv = require('yargs')
    .usage('Usage: $0')
    .alias('p', 'port')
    .alias('v', 'version')
    .help('h')
    .alias('h', 'help')
    .options({
    'p': {
        alias: 'port',
        number: true,
        default: 1511,
        description: 'UDP port'
    },
    'm': {
        alias: 'multicast',
        boolean: true,
        default: true,
        description: 'Bind udp port in multicast'
    },
    'g': {
        alias: 'group',
        string: true,
        description: 'UDP Multicast group, the range is 224.0.0.1 - 239.255.255.255'
    },
    'i': {
        alias: 'interface',
        string: true,
        choices: ['all'].concat(Object.keys(require('os').networkInterfaces())),
        default: 'all',
        description: 'Which network interface udp port bind on.'
    },
    's': {
        alias: 'serialport',
        string: true,
        description: "Xbee's serial port path"
    },
    'l': {
        alias: 'list-serialport',
        boolean: false,
        description: 'List all serial port'
    },
    'r': {
        alias: 'baudrate',
        number: true,
        default: 57600,
        description: 'Xbee baudrate'
    }
})
    .argv;
function listSerialPorts() {
    return __awaiter(this, void 0, void 0, function* () {
        yield SerialPort.list().then((ports) => {
            return ports.filter((port) => {
                return port.manufacturer == 'FTDI';
            }).map((port) => {
                console.log(port.comName);
            });
        });
    });
}
if (argv['list-serialport']) {
    listSerialPorts();
}
else {
    let serialport = null;
    if (argv.serialport) {
        serialport = new SerialPort(argv.serialport, { baudRate: argv.baudrate });
        serialport.on('error', (err) => {
            console.error("Error to open serial port: %s", argv.serialport);
            console.error(err);
        });
        serialport.on('data', (data) => {
            console.log('Serial port Receive: ' + data);
        });
    }
    let optitrack = new server_1.OptitrackServer(argv.port, argv.interface, argv.multicast, argv.group);
    optitrack
        .packets
        .subscribe((packet) => {
        // 这里的 packet 中包含了所有的刚体的数据
        if (packet.dataFrame.rigidBodies.length == 0) {
            return;
        }
        let rb = packet.dataFrame.rigidBodies[0];
        let log = util.format("%s %d %d %d %d %d %d %d %d %d %d", now(), packet.dataFrame.no, packet.dataFrame.timestamp, packet.dataFrame.latency, packet.dataFrame.timecode, rb.x, rb.y, rb.z, rb.qx, rb.qy, rb.qz, rb.qw);
        console.log(log);
        if (serialport) {
            sendToSerialPort(serialport, packet);
        }
    });
    optitrack.start();
}
//# sourceMappingURL=index.js.map