"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dgram = require('dgram');
const os_1 = require("os");
const optitrack_1 = require("optitrack");
const rxjs_1 = require("rxjs");
/**
 * 得到对应网卡的 IP
 */
function getIpOfIface(iface) {
    if (iface == 'all') {
        return null;
    }
    let configs = os_1.networkInterfaces()[iface];
    if (!configs) {
        return null;
    }
    let config = configs.filter((v) => { return v.family == 'IPv4'; })[0];
    if (!config) {
        return null;
    }
    return config.address;
}
exports.getIpOfIface = getIpOfIface;
class OptitrackServer {
    /**
     * port: number 服务器监听的端口
     */
    constructor(port, iface, multicast, group) {
        this.port = port;
        this.multicast = multicast;
        this.group = group;
        /**
         * 用来解析二进制包的解析器
         */
        this.parser = new optitrack_1.OTPacketParser();
        /**
         * 当前是否正在监听中
         */
        this.isListening = false;
        let self = this;
        self.server = dgram.createSocket('udp4');
        self.packets = rxjs_1.Observable.fromEvent(self.server, 'message').map((msg, idx) => {
            let packet = self.parser.parse(msg);
            return packet;
        });
        self.server.on('error', (err) => {
            console.error(`server error:\n${err.stack}`);
            self.stop();
        });
        self.server.on('listening', () => {
            self.isListening = true;
            const address = self.server.address();
            //console.log(`server listening ${address.address}:${address.port}`);
            if (self.multicast && group) {
                self.server.setBroadcast(true);
                self.server.setMulticastLoopback(true);
                self.server.setMulticastTTL(128);
                let ifaceIP = getIpOfIface(iface);
                self.server.addMembership(group, ifaceIP);
            }
        });
    }
    /**
     * 启动 接收 和 解析 的服务器
     */
    start() {
        let self = this;
        self.server.bind(self.port);
    }
    /**
     * 关闭服务器
     */
    stop() {
        let self = this;
        self.isListening = false;
        self.server.close();
    }
}
exports.OptitrackServer = OptitrackServer;
//# sourceMappingURL=server.js.map