const dgram = require('dgram');

import { networkInterfaces, NetworkInterfaceInfo } from 'os';

import { Socket } from 'dgram';
import { OTPacketParser, OTPacket } from 'optitrack';
import { Observable } from 'rxjs';

/**
 * 得到对应网卡的 IP
 */
export function getIpOfIface(iface: string): string {
    if (iface == 'all') {
        return null;
    }

    let configs = networkInterfaces()[iface];

    if (!configs) {
        return null;
    }

    let config = configs.filter((v: NetworkInterfaceInfo) => { return v.family == 'IPv4';})[0];
    if (!config) {
        return null;
    }

    return config.address;
}

export class OptitrackServer {

    /**
     * 用来接收数据包的服务器
     */
    server: Socket;

    /**
     * 用来解析二进制包的解析器
     */
    parser: OTPacketParser = new OTPacketParser();

    /**
     * 用来订阅解析出来的一个个包
     */
    packets: Observable<OTPacket>;

    /**
     * 当前是否正在监听中
     */
    isListening: boolean = false;

    /**
     * port: number 服务器监听的端口
     */
    constructor(public port: number, iface: string, public multicast: boolean, public group: string){
        let self = this;

        self.server = dgram.createSocket('udp4');

        self.packets =  Observable.fromEvent(self.server, 'message').map((msg: Buffer, idx) => {
            let packet = self.parser.parse(msg);
            return packet;
        });

        self.server.on('error', (err) => {
            console.error(`server error:\n${err.stack}`);
            self.stop();
        });

        self.server.on('listening', () => {
            self.isListening = true;
            const address = self.server.address();
            //console.log(`server listening ${address.address}:${address.port}`);
            if (self.multicast && group) {
                self.server.setBroadcast(true);
                self.server.setMulticastLoopback(true);
                self.server.setMulticastTTL(128);

                let ifaceIP = getIpOfIface(iface);
                self.server.addMembership(group, ifaceIP);
            }
        });
    }

    /**
     * 启动 接收 和 解析 的服务器
     */
    start() {
        let self = this;
        self.server.bind(self.port);
    }

    /**
     * 关闭服务器
     */
    stop() {
        let self = this;
        self.isListening = false;
        self.server.close();
    }
}
